@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -54,7 +55,8 @@ public:
                     const Transform3f& tf2) : MotionBase(),
                                               rot(tf1.getQuatRotation()),
                                               trans_start(tf1.getTranslation()),
-                                              trans_range(tf2.getTranslation() - tf1.getTranslation())
+                                              trans_range(tf2.getTranslation() - tf1.getTranslation()),
+                                              tf(tf1)
   {
   }
 
@@ -63,6 +65,7 @@ public:
     rot.fromRotation(R);
     trans_start = T1;
     trans_range = T2 - T1;
+    tf = Transform3f(rot, trans_start);
   }
 
   bool integrate(FCL_REAL dt) const
