@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -51,10 +52,10 @@ class TMatrix3
   
 public:
   TMatrix3();
-  TMatrix3(const boost::shared_ptr<TimeInterval>& time_interval);
+  TMatrix3(const std::shared_ptr<TimeInterval>& time_interval);
   TMatrix3(TaylorModel m[3][3]);
   TMatrix3(const TVector3& v1, const TVector3& v2, const TVector3& v3);
-  TMatrix3(const Matrix3f& m, const boost::shared_ptr<TimeInterval>& time_interval);
+  TMatrix3(const Matrix3f& m, const std::shared_ptr<TimeInterval>& time_interval);
 
   TVector3 getColumn(size_t i) const;
   const TVector3& getRow(size_t i) const;
@@ -98,10 +99,10 @@ public:
   void setZero();
   FCL_REAL diameter() const;
 
-  void setTimeInterval(const boost::shared_ptr<TimeInterval>& time_interval);
+  void setTimeInterval(const std::shared_ptr<TimeInterval>& time_interval);
   void setTimeInterval(FCL_REAL l, FCL_REAL r);
 
-  const boost::shared_ptr<TimeInterval>& getTimeInterval() const;
+  const std::shared_ptr<TimeInterval>& getTimeInterval() const;
 
   TMatrix3& rotationConstrain();
 };
