@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -50,10 +51,10 @@ class TVector3
 public:
   
   TVector3();
-  TVector3(const boost::shared_ptr<TimeInterval>& time_interval);
+  TVector3(const std::shared_ptr<TimeInterval>& time_interval);
   TVector3(TaylorModel v[3]);
   TVector3(const TaylorModel& v0, const TaylorModel& v1, const TaylorModel& v2);
-  TVector3(const Vec3f& v, const boost::shared_ptr<TimeInterval>& time_interval);
+  TVector3(const Vec3f& v, const std::shared_ptr<TimeInterval>& time_interval);
   
   TVector3 operator + (const TVector3& other) const;
   TVector3& operator += (const TVector3& other);
@@ -95,10 +96,10 @@ public:
 
   TaylorModel squareLength() const;
 
-  void setTimeInterval(const boost::shared_ptr<TimeInterval>& time_interval);
+  void setTimeInterval(const std::shared_ptr<TimeInterval>& time_interval);
   void setTimeInterval(FCL_REAL l, FCL_REAL r);
 
-  const boost::shared_ptr<TimeInterval>& getTimeInterval() const;
+  const std::shared_ptr<TimeInterval>& getTimeInterval() const;
 };
 
 void generateTVector3ForLinearFunc(TVector3& v, const Vec3f& position, const Vec3f& velocity);
