@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -42,7 +43,7 @@
 #include "fcl/BV/AABB.h"
 #include "fcl/math/transform.h"
 #include "fcl/ccd/motion_base.h"
-#include <boost/shared_ptr.hpp>
+#include <memory>
 
 namespace fcl
 {
@@ -50,9 +51,9 @@ namespace fcl
 /// @brief object type: BVH (mesh, points), basic geometry, octree
 enum OBJECT_TYPE {OT_UNKNOWN, OT_BVH, OT_GEOM, OT_OCTREE, OT_COUNT};
 
-/// @brief traversal node type: bounding volume (AABB, OBB, RSS, kIOS, OBBRSS, KDOP16, KDOP18, kDOP24), basic shape (box, sphere, capsule, cone, cylinder, convex, plane, triangle), and octree
+/// @brief traversal node type: bounding volume (AABB, OBB, RSS, kIOS, OBBRSS, KDOP16, KDOP18, kDOP24), basic shape (box, sphere, ellipsoid, capsule, cone, cylinder, convex, plane, halfspace, triangle), and octree
 enum NODE_TYPE {BV_UNKNOWN, BV_AABB, BV_OBB, BV_RSS, BV_kIOS, BV_OBBRSS, BV_KDOP16, BV_KDOP18, BV_KDOP24,
-                GEOM_BOX, GEOM_SPHERE, GEOM_CAPSULE, GEOM_CONE, GEOM_CYLINDER, GEOM_CONVEX, GEOM_PLANE, GEOM_HALFSPACE, GEOM_TRIANGLE, GEOM_OCTREE, NODE_COUNT};
+                GEOM_BOX, GEOM_SPHERE, GEOM_ELLIPSOID, GEOM_CAPSULE, GEOM_CONE, GEOM_CYLINDER, GEOM_CONVEX, GEOM_PLANE, GEOM_HALFSPACE, GEOM_TRIANGLE, GEOM_OCTREE, NODE_COUNT};
 
 /// @brief The geometry for the object for collision or distance computation
 class CollisionGeometry
@@ -150,7 +151,7 @@ public:
 class CollisionObject
 {
 public:
- CollisionObject(const boost::shared_ptr<CollisionGeometry> &cgeom_) :
+ CollisionObject(const std::shared_ptr<CollisionGeometry> &cgeom_) :
     cgeom(cgeom_), cgeom_const(cgeom_)
   {
     if (cgeom)
@@ -160,14 +161,14 @@ public:
     }
   }
 
-  CollisionObject(const boost::shared_ptr<CollisionGeometry> &cgeom_, const Transform3f& tf) :
+  CollisionObject(const std::shared_ptr<CollisionGeometry> &cgeom_, const Transform3f& tf) :
     cgeom(cgeom_), cgeom_const(cgeom_), t(tf)
   {
     cgeom->computeLocalAABB();
     computeAABB();
   }
 
-  CollisionObject(const boost::shared_ptr<CollisionGeometry> &cgeom_, const Matrix3f& R, const Vec3f& T):
+  CollisionObject(const std::shared_ptr<CollisionGeometry> &cgeom_, const Matrix3f& R, const Vec3f& T):
       cgeom(cgeom_), cgeom_const(cgeom_), t(Transform3f(R, T))
   {
     cgeom->computeLocalAABB();
@@ -297,13 +298,14 @@ public:
   }
 
   /// @brief get geometry from the object instance
-  const CollisionGeometry* getCollisionGeometry() const FCL_DEPRECATED
+  FCL_DEPRECATED
+  const CollisionGeometry* getCollisionGeometry() const
   {
     return cgeom.get();
   }
 
   /// @brief get geometry from the object instance
-  const boost::shared_ptr<const CollisionGeometry>& collisionGeometry() const
+  const std::shared_ptr<const CollisionGeometry>& collisionGeometry() const
   {
     return cgeom_const;
   }
@@ -340,8 +342,8 @@ public:
 
 protected:
 
-  boost::shared_ptr<CollisionGeometry> cgeom;
-  boost::shared_ptr<const CollisionGeometry> cgeom_const;
+  std::shared_ptr<CollisionGeometry> cgeom;
+  std::shared_ptr<const CollisionGeometry> cgeom_const;
 
   Transform3f t;
 
@@ -357,12 +359,12 @@ protected:
 class ContinuousCollisionObject
 {
 public:
-  ContinuousCollisionObject(const boost::shared_ptr<CollisionGeometry>& cgeom_) :
+  ContinuousCollisionObject(const std::shared_ptr<CollisionGeometry>& cgeom_) :
     cgeom(cgeom_), cgeom_const(cgeom_)
   {
   }
 
-  ContinuousCollisionObject(const boost::shared_ptr<CollisionGeometry>& cgeom_, const boost::shared_ptr<MotionBase>& motion_) :
+  ContinuousCollisionObject(const std::shared_ptr<CollisionGeometry>& cgeom_, const std::shared_ptr<MotionBase>& motion_) :
     cgeom(cgeom_), cgeom_const(cgeom), motion(motion_)
   {
   }
@@ -446,23 +448,24 @@ public:
   }
 
   /// @brief get geometry from the object instance
-  inline const CollisionGeometry* getCollisionGeometry() const FCL_DEPRECATED
+  FCL_DEPRECATED
+  inline const CollisionGeometry* getCollisionGeometry() const
   {
     return cgeom.get();
   }
 
   /// @brief get geometry from the object instance
-  inline const boost::shared_ptr<const CollisionGeometry>& collisionGeometry() const
+  inline const std::shared_ptr<const CollisionGeometry>& collisionGeometry() const
   {
     return cgeom_const;
   }
 
 protected:
 
-  boost::shared_ptr<CollisionGeometry> cgeom;
-  boost::shared_ptr<const CollisionGeometry> cgeom_const;
+  std::shared_ptr<CollisionGeometry> cgeom;
+  std::shared_ptr<const CollisionGeometry> cgeom_const;
 
-  boost::shared_ptr<MotionBase> motion;
+  std::shared_ptr<MotionBase> motion;
 
   /// @brief AABB in the global coordinate for the motion
   mutable AABB aabb;
