@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -38,7 +39,6 @@
 #define FCL_INTERSECT_H
 
 #include "fcl/math/transform.h"
-#include <boost/math/special_functions/erf.hpp>
 
 namespace fcl
 {
@@ -236,7 +236,7 @@ private:
   /// @brief compute the cdf(x) 
   static FCL_REAL gaussianCDF(FCL_REAL x)
   {
-    return 0.5 * boost::math::erfc(-x / sqrt(2.0));
+    return 0.5 * std::erfc(-x / sqrt(2.0));
   }
 
 
