@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -37,6 +38,11 @@
 #ifndef FCL_TRAVERSAL_NODE_OCTREE_H
 #define FCL_TRAVERSAL_NODE_OCTREE_H
 
+#include "fcl/config.h"
+#if not(FCL_HAVE_OCTOMAP)
+#error "This header requires fcl to be compiled with octomap support"
+#endif
+
 #include "fcl/collision_data.h"
 #include "fcl/traversal/traversal_node_base.h"
 #include "fcl/narrowphase/narrowphase.h"
@@ -238,7 +244,7 @@ private:
                                   const S& s, const AABB& aabb2,
                                   const Transform3f& tf1, const Transform3f& tf2) const
   {
-    if(!root1->hasChildren())
+    if(!tree1->nodeHasChildren(root1))
     {
       if(tree1->isNodeOccupied(root1))
       {
@@ -262,9 +268,9 @@ private:
     
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root1->childExists(i))
+      if(tree1->nodeChildExists(root1, i))
       {
-        const OcTree::OcTreeNode* child = root1->getChild(i);
+        const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
         AABB child_bv;
         computeChildBV(bv1, i, child_bv);
         
@@ -297,20 +303,20 @@ private:
         Transform3f box_tf;
         constructBox(bv1, tf1, box, box_tf);
 
-        if(solver->shapeIntersect(box, box_tf, s, tf2, NULL, NULL, NULL))
+        if(solver->shapeIntersect(box, box_tf, s, tf2, NULL))
         {
           AABB overlap_part;
           AABB aabb1, aabb2;
           computeBV<AABB, Box>(box, box_tf, aabb1);
           computeBV<AABB, S>(s, tf2, aabb2);
           aabb1.overlap(aabb2, overlap_part);
-	  cresult->addCostSource(CostSource(overlap_part, tree1->getOccupancyThres() * s.cost_density), crequest->num_max_cost_sources);          
+          cresult->addCostSource(CostSource(overlap_part, tree1->getOccupancyThres() * s.cost_density), crequest->num_max_cost_sources);
         }
       }
 
       return false;
     }
-    else if(!root1->hasChildren())
+    else if(!tree1->nodeHasChildren(root1))
     {
       if(tree1->isNodeOccupied(root1) && s.isOccupied()) // occupied area
       {
@@ -325,7 +331,7 @@ private:
           bool is_intersect = false;
           if(!crequest->enable_contact)
           {
-            if(solver->shapeIntersect(box, box_tf, s, tf2, NULL, NULL, NULL))
+            if(solver->shapeIntersect(box, box_tf, s, tf2, NULL))
             {
               is_intersect = true;
               if(cresult->numContacts() < crequest->num_max_contacts)
@@ -334,15 +340,29 @@ private:
           }
           else
           {
-            Vec3f contact;
-            FCL_REAL depth;
-            Vec3f normal;
-
-            if(solver->shapeIntersect(box, box_tf, s, tf2, &contact, &depth, &normal))
+            std::vector<ContactPoint> contacts;
+            if(solver->shapeIntersect(box, box_tf, s, tf2, &contacts))
             {
               is_intersect = true;
-              if(cresult->numContacts() < crequest->num_max_contacts)
-                cresult->addContact(Contact(tree1, &s, root1 - tree1->getRoot(), Contact::NONE, contact, normal, depth));
+              if(crequest->num_max_contacts > cresult->numContacts())
+              {
+                const size_t free_space = crequest->num_max_contacts - cresult->numContacts();
+                size_t num_adding_contacts;
+
+                // If the free space is not enough to add all the new contacts, we add contacts in descent order of penetration depth.
+                if (free_space < contacts.size())
+                {
+                  std::partial_sort(contacts.begin(), contacts.begin() + free_space, contacts.end(), std::bind(comparePenDepth, std::placeholders::_2, std::placeholders::_1));
+                  num_adding_contacts = free_space;
+                }
+                else
+                {
+                  num_adding_contacts = contacts.size();
+                }
+
+                for(size_t i = 0; i < num_adding_contacts; ++i)
+                  cresult->addContact(Contact(tree1, &s, root1 - tree1->getRoot(), Contact::NONE, contacts[i].pos, contacts[i].normal, contacts[i].penetration_depth));
+              }
             }
           }
 
@@ -369,7 +389,7 @@ private:
           Transform3f box_tf;
           constructBox(bv1, tf1, box, box_tf);
 
-          if(solver->shapeIntersect(box, box_tf, s, tf2, NULL, NULL, NULL))
+          if(solver->shapeIntersect(box, box_tf, s, tf2, NULL))
           {
             AABB overlap_part;
             AABB aabb1, aabb2;
@@ -399,9 +419,9 @@ private:
 
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root1->childExists(i))
+      if(tree1->nodeChildExists(root1, i))
       {
-        const OcTree::OcTreeNode* child = root1->getChild(i);
+        const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
         AABB child_bv;
         computeChildBV(bv1, i, child_bv);
         
@@ -426,7 +446,7 @@ private:
                                  const BVHModel<BV>* tree2, int root2,
                                  const Transform3f& tf1, const Transform3f& tf2) const
   {
-    if(!root1->hasChildren() && tree2->getBV(root2).isLeaf())
+    if(!tree1->nodeHasChildren(root1) && tree2->getBV(root2).isLeaf())
     {
       if(tree1->isNodeOccupied(root1))
       {
@@ -454,13 +474,13 @@ private:
 
     if(!tree1->isNodeOccupied(root1)) return false;
 
-    if(tree2->getBV(root2).isLeaf() || (root1->hasChildren() && (bv1.size() > tree2->getBV(root2).bv.size())))
+    if(tree2->getBV(root2).isLeaf() || (tree1->nodeHasChildren(root1) && (bv1.size() > tree2->getBV(root2).bv.size())))
     {
       for(unsigned int i = 0; i < 8; ++i)
       {
-        if(root1->childExists(i))
+        if(tree1->nodeChildExists(root1, i))
         {
-          const OcTree::OcTreeNode* child = root1->getChild(i);
+          const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
           AABB child_bv;
           computeChildBV(bv1, i, child_bv);
 
@@ -556,7 +576,7 @@ private:
         return false;
       }
     }
-    else if(!root1->hasChildren() && tree2->getBV(root2).isLeaf())
+    else if(!tree1->nodeHasChildren(root1) && tree2->getBV(root2).isLeaf())
     {
       if(tree1->isNodeOccupied(root1) && tree2->isOccupied())
       {
@@ -582,7 +602,7 @@ private:
             {
               is_intersect = true;
               if(cresult->numContacts() < crequest->num_max_contacts)
-                cresult->addContact(Contact(tree1, tree2, root1 - tree1->getRoot(), root2));
+                cresult->addContact(Contact(tree1, tree2, root1 - tree1->getRoot(), primitive_id));
             }
           }
           else
@@ -595,7 +615,7 @@ private:
             {
               is_intersect = true;
               if(cresult->numContacts() < crequest->num_max_contacts)
-                cresult->addContact(Contact(tree1, tree2, root1 - tree1->getRoot(), root2, contact, normal, depth));
+                cresult->addContact(Contact(tree1, tree2, root1 - tree1->getRoot(), primitive_id, contact, normal, depth));
             }
           }
 
@@ -661,13 +681,13 @@ private:
       if(!obb1.overlap(obb2)) return false;      
     }
    
-    if(tree2->getBV(root2).isLeaf() || (root1->hasChildren() && (bv1.size() > tree2->getBV(root2).bv.size())))
+    if(tree2->getBV(root2).isLeaf() || (tree1->nodeHasChildren(root1) && (bv1.size() > tree2->getBV(root2).bv.size())))
     {
       for(unsigned int i = 0; i < 8; ++i)
       {
-        if(root1->childExists(i))
+        if(tree1->nodeChildExists(root1, i))
         {
-          const OcTree::OcTreeNode* child = root1->getChild(i);
+          const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
           AABB child_bv;
           computeChildBV(bv1, i, child_bv);
           
@@ -701,7 +721,7 @@ private:
                              const OcTree* tree2, const OcTree::OcTreeNode* root2, const AABB& bv2,
                              const Transform3f& tf1, const Transform3f& tf2) const
   {
-    if(!root1->hasChildren() && !root2->hasChildren())
+    if(!tree1->nodeHasChildren(root1) && !tree2->nodeHasChildren(root2))
     {
       if(tree1->isNodeOccupied(root1) && tree2->isNodeOccupied(root2))
       {
@@ -724,13 +744,13 @@ private:
 
     if(!tree1->isNodeOccupied(root1) || !tree2->isNodeOccupied(root2)) return false;
 
-    if(!root2->hasChildren() || (root1->hasChildren() && (bv1.size() > bv2.size())))
+    if(!tree2->nodeHasChildren(root2) || (tree1->nodeHasChildren(root1) && (bv1.size() > bv2.size())))
     {
       for(unsigned int i = 0; i < 8; ++i)
       {
-        if(root1->childExists(i))
+        if(tree1->nodeChildExists(root1, i))
         {
-          const OcTree::OcTreeNode* child = root1->getChild(i);
+          const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
           AABB child_bv;
           computeChildBV(bv1, i, child_bv);
 
@@ -753,9 +773,9 @@ private:
     {
       for(unsigned int i = 0; i < 8; ++i)
       {
-        if(root2->childExists(i))
+        if(tree2->nodeChildExists(root2, i))
         {
-          const OcTree::OcTreeNode* child = root2->getChild(i);
+          const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
           AABB child_bv;
           computeChildBV(bv2, i, child_bv);
 
@@ -807,13 +827,13 @@ private:
     }
     else if(!root1 && root2)
     {
-      if(root2->hasChildren())
+      if(tree2->nodeHasChildren(root2))
       {
         for(unsigned int i = 0; i < 8; ++i)
         {
-          if(root2->childExists(i))
+          if(tree2->nodeChildExists(root2, i))
           {
-            const OcTree::OcTreeNode* child = root2->getChild(i);
+            const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
             AABB child_bv;
             computeChildBV(bv2, i, child_bv);
             if(OcTreeIntersectRecurse(tree1, NULL, bv1, tree2, child, child_bv, tf1, tf2))
@@ -838,13 +858,13 @@ private:
     }
     else if(root1 && !root2)
     {
-      if(root1->hasChildren())
+      if(tree1->nodeHasChildren(root1))
       {
         for(unsigned int i = 0; i < 8; ++i)
         {
-          if(root1->childExists(i))
+          if(tree1->nodeChildExists(root1, i))
           {
-            const OcTree::OcTreeNode* child = root1->getChild(i);
+            const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
             AABB child_bv;
             computeChildBV(bv1, i,  child_bv);
             if(OcTreeIntersectRecurse(tree1, child, child_bv, tree2, NULL, bv2, tf1, tf2))
@@ -867,7 +887,7 @@ private:
       
       return false;
     }
-    else if(!root1->hasChildren() && !root2->hasChildren())
+    else if(!tree1->nodeHasChildren(root1) && !tree2->nodeHasChildren(root2))
     {
       if(tree1->isNodeOccupied(root1) && tree2->isNodeOccupied(root2)) // occupied area
       {
@@ -892,14 +912,29 @@ private:
           constructBox(bv1, tf1, box1, box1_tf);
           constructBox(bv2, tf2, box2, box2_tf);
 
-          Vec3f contact;
-          FCL_REAL depth;
-          Vec3f normal;
-          if(solver->shapeIntersect(box1, box1_tf, box2, box2_tf, &contact, &depth, &normal))
+          std::vector<ContactPoint> contacts;
+          if(solver->shapeIntersect(box1, box1_tf, box2, box2_tf, &contacts))
           {
             is_intersect = true;
-            if(cresult->numContacts() < crequest->num_max_contacts)
-              cresult->addContact(Contact(tree1, tree2, root1 - tree1->getRoot(), root2 - tree2->getRoot(), contact, normal, depth));
+            if(crequest->num_max_contacts > cresult->numContacts())
+            {
+              const size_t free_space = crequest->num_max_contacts - cresult->numContacts();
+              size_t num_adding_contacts;
+
+              // If the free space is not enough to add all the new contacts, we add contacts in descent order of penetration depth.
+              if (free_space < contacts.size())
+              {
+                std::partial_sort(contacts.begin(), contacts.begin() + free_space, contacts.end(), std::bind(comparePenDepth, std::placeholders::_2, std::placeholders::_1));
+                num_adding_contacts = free_space;
+              }
+              else
+              {
+                num_adding_contacts = contacts.size();
+              }
+
+              for(size_t i = 0; i < num_adding_contacts; ++i)
+                cresult->addContact(Contact(tree1, tree2, root1 - tree1->getRoot(), root2 - tree2->getRoot(), contacts[i].pos, contacts[i].normal, contacts[i].penetration_depth));
+            }
           }
         }
 
@@ -960,13 +995,13 @@ private:
       if(!obb1.overlap(obb2)) return false;
     }
 
-    if(!root2->hasChildren() || (root1->hasChildren() && (bv1.size() > bv2.size())))
+    if(!tree2->nodeHasChildren(root2) || (tree1->nodeHasChildren(root1) && (bv1.size() > bv2.size())))
     {
       for(unsigned int i = 0; i < 8; ++i)
       {
-        if(root1->childExists(i))
+        if(tree1->nodeChildExists(root1, i))
         {
-          const OcTree::OcTreeNode* child = root1->getChild(i);
+          const OcTree::OcTreeNode* child = tree1->getNodeChild(root1, i);
           AABB child_bv;
           computeChildBV(bv1, i, child_bv);
         
@@ -991,9 +1026,9 @@ private:
     {
       for(unsigned int i = 0; i < 8; ++i)
       {
-        if(root2->childExists(i))
+        if(tree2->nodeChildExists(root2, i))
         {
-          const OcTree::OcTreeNode* child = root2->getChild(i);
+          const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
           AABB child_bv;
           computeChildBV(bv2, i, child_bv);
           
