@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -37,7 +38,7 @@
 #include "fcl/broadphase/broadphase_SaP.h"
 #include <algorithm>
 #include <limits>
-#include <boost/bind.hpp>
+#include <functional>
 
 namespace fcl
 {
@@ -89,6 +90,8 @@ void SaPCollisionManager::unregisterObject(CollisionObject* obj)
 
 void SaPCollisionManager::registerObjects(const std::vector<CollisionObject*>& other_objs)
 {
+  if(other_objs.empty()) return;
+
   if(size() > 0)
     BroadPhaseCollisionManager::registerObjects(other_objs);
   else
@@ -117,9 +120,9 @@ void SaPCollisionManager::registerObjects(const std::vector<CollisionObject*>& o
     for(size_t coord = 0; coord < 3; ++coord)
     { 
       std::sort(endpoints.begin(), endpoints.end(), 
-                boost::bind(std::less<Vec3f::U>(),
-                            boost::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const >(&EndPoint::getVal), _1, coord),
-                            boost::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const >(&EndPoint::getVal), _2, coord)));
+                std::bind(std::less<Vec3f::U>(),
+                            std::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const >(&EndPoint::getVal), std::placeholders::_1, coord),
+                            std::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const >(&EndPoint::getVal), std::placeholders::_2, coord)));
 
       endpoints[0]->prev[coord] = NULL;
       endpoints[0]->next[coord] = endpoints[1];
@@ -272,6 +275,8 @@ void SaPCollisionManager::registerObject(CollisionObject* obj)
 
 void SaPCollisionManager::setup()
 {
+  if(size() == 0) return;
+
   FCL_REAL scale[3];
   scale[0] = (velist[0].back())->getVal(0) - velist[0][0]->getVal(0);
   scale[1] = (velist[1].back())->getVal(1) - velist[1][0]->getVal(1);;
@@ -508,9 +513,9 @@ bool SaPCollisionManager::collide_(CollisionObject* obj, void* cdata, CollisionC
   
   // compute stop_pos by binary search, this is cheaper than check it in while iteration linearly
   std::vector<EndPoint*>::const_iterator res_it = std::upper_bound(velist[axis].begin(), velist[axis].end(), &dummy,
-                                                                   boost::bind(std::less<Vec3f::U>(),
-                                                                               boost::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), _1, axis),
-                                                                               boost::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), _2, axis)));
+                                                                   std::bind(std::less<Vec3f::U>(),
+                                                                               std::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), std::placeholders::_1, axis),
+                                                                               std::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), std::placeholders::_2, axis)));
   
   EndPoint* end_pos = NULL;
   if(res_it != velist[axis].end())
@@ -574,9 +579,9 @@ bool SaPCollisionManager::distance_(CollisionObject* obj, void* cdata, DistanceC
     
  
     std::vector<EndPoint*>::const_iterator res_it = std::upper_bound(velist[axis].begin(), velist[axis].end(), &dummy,
-                                                                     boost::bind(std::less<Vec3f::U>(),
-                                                                                 boost::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), _1, axis),
-                                                                                 boost::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), _2, axis)));
+                                                                     std::bind(std::less<Vec3f::U>(),
+                                                                                 std::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), std::placeholders::_1, axis),
+                                                                                 std::bind(static_cast<Vec3f::U (EndPoint::*)(size_t) const>(&EndPoint::getVal), std::placeholders::_2, axis)));
 
     EndPoint* end_pos = NULL;
     if(res_it != velist[axis].end())
