@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -73,7 +74,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
 
           box->cost_density = tree2->getDefaultOccupancy();
           
-          CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+          CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
           return callback(obj1, &obj2, cdata);
         }
       }
@@ -88,7 +89,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
     
     return false;
   }
-  else if(root1->isLeaf() && !root2->hasChildren())
+  else if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     CollisionObject* obj1 = static_cast<CollisionObject*>(root1->data);
 
@@ -107,7 +108,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
         box->cost_density = root2->getOccupancy();
         box->threshold_occupied = tree2->getOccupancyThres();
 
-        CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+        CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
         return callback(obj1, &obj2, cdata);
       }
       else return false;
@@ -121,7 +122,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
 
   if(tree2->isNodeFree(root2) || !obb1.overlap(obb2)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     if(collisionRecurse_(root1->children[0], tree2, root2, root2_bv, tf2, cdata, callback))
       return true;
@@ -132,9 +133,9 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -172,7 +173,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
 
           box->cost_density = tree2->getOccupancyThres(); // thresholds are 0, 1, so uncertain
 
-          CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+          CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
           return callback(obj1, &obj2, cdata);
         }
       }
@@ -187,7 +188,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
 
     return false;
   }
-  else if(root1->isLeaf() && !root2->hasChildren())
+  else if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     CollisionObject* obj1 = static_cast<CollisionObject*>(root1->data);
   
@@ -203,7 +204,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
         box->cost_density = root2->getOccupancy();
         box->threshold_occupied = tree2->getOccupancyThres();
 
-        CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+        CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
         return callback(obj1, &obj2, cdata);
       }
       else return false;
@@ -214,7 +215,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
   const AABB& root2_bv_t = translate(root2_bv, tf2);
   if(tree2->isNodeFree(root2) || !root1->bv.overlap(root2_bv_t)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     if(collisionRecurse_(root1->children[0], tree2, root2, root2_bv, tf2, cdata, callback))
       return true;
@@ -225,9 +226,9 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -249,14 +250,14 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1,
 
 bool distanceRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, const OcTree* tree2, const OcTree::OcTreeNode* root2, const AABB& root2_bv, const Transform3f& tf2, void* cdata, DistanceCallBack callback, FCL_REAL& min_dist)
 {
-  if(root1->isLeaf() && !root2->hasChildren())
+  if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     if(tree2->isNodeOccupied(root2))
     {
       Box* box = new Box();
       Transform3f box_tf;
       constructBox(root2_bv, tf2, *box, box_tf);
-      CollisionObject obj(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+      CollisionObject obj(std::shared_ptr<CollisionGeometry>(box), box_tf);
       return callback(static_cast<CollisionObject*>(root1->data), &obj, cdata, min_dist);
     }
     else return false;
@@ -264,7 +265,7 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, c
   
   if(!tree2->isNodeOccupied(root2)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     AABB aabb2;
     convertBV(root2_bv, tf2, aabb2);
@@ -305,9 +306,9 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, c
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -338,14 +339,14 @@ bool collisionRecurse(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, c
 
 bool distanceRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, const OcTree* tree2, const OcTree::OcTreeNode* root2, const AABB& root2_bv, const Vec3f& tf2, void* cdata, DistanceCallBack callback, FCL_REAL& min_dist)
 {
-  if(root1->isLeaf() && !root2->hasChildren())
+  if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     if(tree2->isNodeOccupied(root2))
     {
       Box* box = new Box();
       Transform3f box_tf;
       constructBox(root2_bv, tf2, *box, box_tf);
-      CollisionObject obj(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+      CollisionObject obj(std::shared_ptr<CollisionGeometry>(box), box_tf);
       return callback(static_cast<CollisionObject*>(root1->data), &obj, cdata, min_dist);
     }
     else return false;
@@ -353,7 +354,7 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, c
   
   if(!tree2->isNodeOccupied(root2)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     const AABB& aabb2 = translate(root2_bv, tf2);
     FCL_REAL d1 = aabb2.distance(root1->children[0]->bv);
@@ -392,9 +393,9 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager::DynamicAABBNode* root1, c
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
         const AABB& aabb2 = translate(child_bv, tf2);
@@ -634,6 +635,8 @@ bool selfDistanceRecurse(DynamicAABBTreeCollisionManager::DynamicAABBNode* root,
 
 void DynamicAABBTreeCollisionManager::registerObjects(const std::vector<CollisionObject*>& other_objs)
 {
+  if(other_objs.empty()) return;
+
   if(size() > 0)
   {
     BroadPhaseCollisionManager::registerObjects(other_objs);
@@ -746,7 +749,7 @@ void DynamicAABBTreeCollisionManager::collide(CollisionObject* obj, void* cdata,
     {
       if(!octree_as_geometry_collide)
       {
-        const OcTree* octree = static_cast<const OcTree*>(obj->getCollisionGeometry());
+        const OcTree* octree = static_cast<const OcTree*>(obj->collisionGeometry().get());
         details::dynamic_AABB_tree::collisionRecurse(dtree.getRoot(), octree, octree->getRoot(), octree->getRootBV(), obj->getTransform(), cdata, callback); 
       }
       else
@@ -770,7 +773,7 @@ void DynamicAABBTreeCollisionManager::distance(CollisionObject* obj, void* cdata
     {
       if(!octree_as_geometry_distance)
       {
-        const OcTree* octree = static_cast<const OcTree*>(obj->getCollisionGeometry());
+        const OcTree* octree = static_cast<const OcTree*>(obj->collisionGeometry().get());
         details::dynamic_AABB_tree::distanceRecurse(dtree.getRoot(), octree, octree->getRoot(), octree->getRootBV(), obj->getTransform(), cdata, callback, min_dist);
       }
       else
