@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -73,7 +74,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
         
           box->cost_density = tree2->getDefaultOccupancy();
 
-          CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+          CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
           return callback(obj1, &obj2, cdata);
         }
       }
@@ -88,7 +89,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
 
     return false;
   }
-  else if(root1->isLeaf() && !root2->hasChildren())
+  else if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     CollisionObject* obj1 = static_cast<CollisionObject*>(root1->data);
     if(!tree2->isNodeFree(root2) && !obj1->isFree())
@@ -106,7 +107,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
         box->cost_density = root2->getOccupancy();
         box->threshold_occupied = tree2->getOccupancyThres();
 
-        CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+        CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
         return callback(obj1, &obj2, cdata);
       }
       else return false;
@@ -120,7 +121,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
   
   if(tree2->isNodeFree(root2) || !obb1.overlap(obb2)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     if(collisionRecurse_(nodes1, root1->children[0], tree2, root2, root2_bv, tf2, cdata, callback))
       return true;
@@ -131,9 +132,9 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -174,7 +175,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
 
           box->cost_density = tree2->getDefaultOccupancy();
 
-          CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+          CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
           return callback(obj1, &obj2, cdata);
         }
       }
@@ -189,7 +190,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
 
     return false;
   }
-  else if(root1->isLeaf() && !root2->hasChildren())
+  else if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     CollisionObject* obj1 = static_cast<CollisionObject*>(root1->data);
     if(!tree2->isNodeFree(root2) && !obj1->isFree())
@@ -204,7 +205,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
         box->cost_density = root2->getOccupancy();
         box->threshold_occupied = tree2->getOccupancyThres();
 
-        CollisionObject obj2(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+        CollisionObject obj2(std::shared_ptr<CollisionGeometry>(box), box_tf);
         return callback(obj1, &obj2, cdata);
       }
       else return false;
@@ -215,7 +216,7 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
   const AABB& root_bv_t = translate(root2_bv, tf2);
   if(tree2->isNodeFree(root2) || !root1->bv.overlap(root_bv_t)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     if(collisionRecurse_(nodes1, root1->children[0], tree2, root2, root2_bv, tf2, cdata, callback))
       return true;
@@ -226,9 +227,9 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -252,14 +253,14 @@ bool collisionRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* n
 bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* nodes1, size_t root1_id, const OcTree* tree2, const OcTree::OcTreeNode* root2, const AABB& root2_bv, const Transform3f& tf2, void* cdata, DistanceCallBack callback, FCL_REAL& min_dist)
 {
   DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* root1 = nodes1 + root1_id;
-  if(root1->isLeaf() && !root2->hasChildren())
+  if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     if(tree2->isNodeOccupied(root2))
     {
       Box* box = new Box();
       Transform3f box_tf;
       constructBox(root2_bv, tf2, *box, box_tf);
-      CollisionObject obj(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+      CollisionObject obj(std::shared_ptr<CollisionGeometry>(box), box_tf);
       return callback(static_cast<CollisionObject*>(root1->data), &obj, cdata, min_dist);
     }
     else return false;
@@ -267,7 +268,7 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* no
 
   if(!tree2->isNodeOccupied(root2)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     AABB aabb2;
     convertBV(root2_bv, tf2, aabb2);
@@ -308,9 +309,9 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* no
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -334,14 +335,14 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* no
 bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* nodes1, size_t root1_id, const OcTree* tree2, const OcTree::OcTreeNode* root2, const AABB& root2_bv, const Vec3f& tf2, void* cdata, DistanceCallBack callback, FCL_REAL& min_dist)
 {
   DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* root1 = nodes1 + root1_id;
-  if(root1->isLeaf() && !root2->hasChildren())
+  if(root1->isLeaf() && !tree2->nodeHasChildren(root2))
   {
     if(tree2->isNodeOccupied(root2))
     {
       Box* box = new Box();
       Transform3f box_tf;
       constructBox(root2_bv, tf2, *box, box_tf);
-      CollisionObject obj(boost::shared_ptr<CollisionGeometry>(box), box_tf);
+      CollisionObject obj(std::shared_ptr<CollisionGeometry>(box), box_tf);
       return callback(static_cast<CollisionObject*>(root1->data), &obj, cdata, min_dist);
     }
     else return false;
@@ -349,7 +350,7 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* no
 
   if(!tree2->isNodeOccupied(root2)) return false;
 
-  if(!root2->hasChildren() || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
+  if(!tree2->nodeHasChildren(root2) || (!root1->isLeaf() && (root1->bv.size() > root2_bv.size())))
   {
     const AABB& aabb2 = translate(root2_bv, tf2);
 
@@ -389,9 +390,9 @@ bool distanceRecurse_(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* no
   {
     for(unsigned int i = 0; i < 8; ++i)
     {
-      if(root2->childExists(i))
+      if(tree2->nodeChildExists(root2, i))
       {
-        const OcTree::OcTreeNode* child = root2->getChild(i);
+        const OcTree::OcTreeNode* child = tree2->getNodeChild(root2, i);
         AABB child_bv;
         computeChildBV(root2_bv, i, child_bv);
 
@@ -657,6 +658,8 @@ bool distanceRecurse(DynamicAABBTreeCollisionManager_Array::DynamicAABBNode* nod
 
 void DynamicAABBTreeCollisionManager_Array::registerObjects(const std::vector<CollisionObject*>& other_objs)
 {
+  if(other_objs.empty()) return;
+
   if(size() > 0)
   {
     BroadPhaseCollisionManager::registerObjects(other_objs);
@@ -771,7 +774,7 @@ void DynamicAABBTreeCollisionManager_Array::collide(CollisionObject* obj, void*
     {
       if(!octree_as_geometry_collide)
       {
-        const OcTree* octree = static_cast<const OcTree*>(obj->getCollisionGeometry());
+        const OcTree* octree = static_cast<const OcTree*>(obj->collisionGeometry().get());
         details::dynamic_AABB_tree_array::collisionRecurse(dtree.getNodes(), dtree.getRoot(), octree, octree->getRoot(), octree->getRootBV(), obj->getTransform(), cdata, callback); 
       }
       else
@@ -795,7 +798,7 @@ void DynamicAABBTreeCollisionManager_Array::distance(CollisionObject* obj, void*
     {
       if(!octree_as_geometry_distance)
       {
-        const OcTree* octree = static_cast<const OcTree*>(obj->getCollisionGeometry());
+        const OcTree* octree = static_cast<const OcTree*>(obj->collisionGeometry().get());
         details::dynamic_AABB_tree_array::distanceRecurse(dtree.getNodes(), dtree.getRoot(), octree, octree->getRoot(), octree->getRootBV(), obj->getTransform(), cdata, callback, min_dist);
       }
       else
