@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -38,7 +39,6 @@
 #include <iostream>
 #include <limits>
 #include <vector>
-#include <boost/math/special_functions/fpclassify.hpp>
 
 namespace fcl
 {
@@ -1199,7 +1199,7 @@ void TriangleDistance::segPoints(const Vec3f& P, const Vec3f& A, const Vec3f& Q,
 
   // clamp result so t is on the segment P,A
 
-  if((t < 0) || boost::math::isnan(t)) t = 0; else if(t > 1) t = 1;
+  if((t < 0) || std::isnan(t)) t = 0; else if(t > 1) t = 1;
 
   // find u for point on ray Q,B closest to point at t
 
@@ -1209,13 +1209,13 @@ void TriangleDistance::segPoints(const Vec3f& P, const Vec3f& A, const Vec3f& Q,
   // closest points, otherwise, clamp u, recompute and
   // clamp t
 
-  if((u <= 0) || boost::math::isnan(u))
+  if((u <= 0) || std::isnan(u))
   {
     Y = Q;
 
     t = A_dot_T / A_dot_A;
 
-    if((t <= 0) || boost::math::isnan(t))
+    if((t <= 0) || std::isnan(t))
     {
       X = P;
       VEC = Q - P;
@@ -1238,7 +1238,7 @@ void TriangleDistance::segPoints(const Vec3f& P, const Vec3f& A, const Vec3f& Q,
 
     t = (A_dot_B + A_dot_T) / A_dot_A;
 
-    if((t <= 0) || boost::math::isnan(t))
+    if((t <= 0) || std::isnan(t))
     {
       X = P;
       VEC = Y - P;
@@ -1260,7 +1260,7 @@ void TriangleDistance::segPoints(const Vec3f& P, const Vec3f& A, const Vec3f& Q,
   {
     Y = Q + B * u;
 
-    if((t <= 0) || boost::math::isnan(t))
+    if((t <= 0) || std::isnan(t))
     {
       X = P;
       TMP = T.cross(B);
