@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -34,9 +35,9 @@
 
 /** \author Jia Pan */
 
-
+#include "fcl/math/constants.h"
 #include "fcl/math/transform.h"
-#include <boost/math/constants/constants.hpp>
+#include <cassert>
 
 namespace fcl
 {
@@ -350,17 +351,17 @@ void Quaternion3f::toEuler(FCL_REAL& a, FCL_REAL& b, FCL_REAL& c) const
   b = asin(-R(2, 0));
   c = atan2(R(2, 1), R(2, 2));
 
-  if(b == boost::math::constants::pi<double>() * 0.5)
+  if(b == constants::pi * 0.5)
   {
     if(a > 0)
-      a -= boost::math::constants::pi<double>();
+      a -= constants::pi;
     else 
-      a += boost::math::constants::pi<double>();
+      a += constants::pi;
 
     if(c > 0)
-      c -= boost::math::constants::pi<double>();
+      c -= constants::pi;
     else
-      c += boost::math::constants::pi<double>();
+      c += constants::pi;
   }
 }
 
@@ -410,7 +411,7 @@ Vec3f Quaternion3f::getRow(std::size_t i) const
 
 const Matrix3f& Transform3f::getRotationInternal() const
 {
-  boost::mutex::scoped_lock slock(const_cast<boost::mutex&>(lock_));
+  std::unique_lock<std::mutex> slock(const_cast<std::mutex&>(lock_));
   if(!matrix_set)
   {
     q.toRotation(R);
