@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -83,6 +84,20 @@ Vec3f getSupport(const ShapeBase* shape, const Vec3f& dir)
       return dir * sphere->radius;
     }
     break;
+  case GEOM_ELLIPSOID:
+    {
+      const Ellipsoid* ellipsoid = static_cast<const Ellipsoid*>(shape);
+
+      const FCL_REAL a2 = ellipsoid->radii[0] * ellipsoid->radii[0];
+      const FCL_REAL b2 = ellipsoid->radii[1] * ellipsoid->radii[1];
+      const FCL_REAL c2 = ellipsoid->radii[2] * ellipsoid->radii[2];
+
+      const Vec3f v(a2 * dir[0], b2 * dir[1], c2 * dir[2]);
+      const FCL_REAL d = std::sqrt(v.dot(dir));
+
+      return v / d;
+    }
+    break;
   case GEOM_CAPSULE:
     {
       const Capsule* capsule = static_cast<const Capsule*>(shape);
