@@ -1,7 +1,9 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
+ *  Copyright (c) 2016, Toyota Research Institute
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +16,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -93,11 +95,11 @@ BOOST_AUTO_TEST_CASE(vec_test_basic_vec32)
   v1 = Vec3f32(1.0f, 2.0f, 3.0f);
   v2 = Vec3f32(3.0f, 4.0f, 5.0f);
   BOOST_CHECK((v1.cross(v2)).equal(Vec3f32(-2.0f, 4.0f, -2.0f)));
-  BOOST_CHECK(abs(v1.dot(v2) - 26) < 1e-5);
+  BOOST_CHECK(std::abs(v1.dot(v2) - 26) < 1e-5);
 
   v1 = Vec3f32(3.0f, 4.0f, 5.0f);
-  BOOST_CHECK(abs(v1.sqrLength() - 50.0) < 1e-5);
-  BOOST_CHECK(abs(v1.length() - sqrt(50.0)) < 1e-5);
+  BOOST_CHECK(std::abs(v1.sqrLength() - 50.0) < 1e-5);
+  BOOST_CHECK(std::abs(v1.length() - sqrt(50.0)) < 1e-5);
   BOOST_CHECK(normalize(v1).equal(v1 / v1.length()));
 }
 
@@ -148,11 +150,11 @@ BOOST_AUTO_TEST_CASE(vec_test_basic_vec64)
   v1 = Vec3f64(1.0, 2.0, 3.0);
   v2 = Vec3f64(3.0, 4.0, 5.0);
   BOOST_CHECK((v1.cross(v2)).equal(Vec3f64(-2.0, 4.0, -2.0)));
-  BOOST_CHECK(abs(v1.dot(v2) - 26) < 1e-5);
+  BOOST_CHECK(std::abs(v1.dot(v2) - 26) < 1e-5);
 
   v1 = Vec3f64(3.0, 4.0, 5.0);
-  BOOST_CHECK(abs(v1.sqrLength() - 50.0) < 1e-5);
-  BOOST_CHECK(abs(v1.length() - sqrt(50.0)) < 1e-5);
+  BOOST_CHECK(std::abs(v1.sqrLength() - 50.0) < 1e-5);
+  BOOST_CHECK(std::abs(v1.length() - sqrt(50.0)) < 1e-5);
   BOOST_CHECK(normalize(v1).equal(v1 / v1.length()));
 
 
@@ -211,11 +213,11 @@ BOOST_AUTO_TEST_CASE(vec_test_sse_vec32)
   v1 = Vec3f32(1.0f, 2.0f, 3.0f);
   v2 = Vec3f32(3.0f, 4.0f, 5.0f);
   BOOST_CHECK((v1.cross(v2)).equal(Vec3f32(-2.0f, 4.0f, -2.0f)));
-  BOOST_CHECK(abs(v1.dot(v2) - 26) < 1e-5);
+  BOOST_CHECK(std::abs(v1.dot(v2) - 26) < 1e-5);
 
   v1 = Vec3f32(3.0f, 4.0f, 5.0f);
-  BOOST_CHECK(abs(v1.sqrLength() - 50) < 1e-5);
-  BOOST_CHECK(abs(v1.length() - sqrt(50)) < 1e-5);
+  BOOST_CHECK(std::abs(v1.sqrLength() - 50) < 1e-5);
+  BOOST_CHECK(std::abs(v1.length() - sqrt(50)) < 1e-5);
   BOOST_CHECK(normalize(v1).equal(v1 / v1.length()));
 }
 
@@ -266,11 +268,11 @@ BOOST_AUTO_TEST_CASE(vec_test_sse_vec64)
   v1 = Vec3f64(1.0, 2.0, 3.0);
   v2 = Vec3f64(3.0, 4.0, 5.0);
   BOOST_CHECK((v1.cross(v2)).equal(Vec3f64(-2.0, 4.0, -2.0)));
-  BOOST_CHECK(abs(v1.dot(v2) - 26) < 1e-5);
+  BOOST_CHECK(std::abs(v1.dot(v2) - 26) < 1e-5);
 
   v1 = Vec3f64(3.0, 4.0, 5.0);
-  BOOST_CHECK(abs(v1.sqrLength() - 50) < 1e-5);
-  BOOST_CHECK(abs(v1.length() - sqrt(50)) < 1e-5);
+  BOOST_CHECK(std::abs(v1.sqrLength() - 50) < 1e-5);
+  BOOST_CHECK(std::abs(v1.length() - sqrt(50)) < 1e-5);
   BOOST_CHECK(normalize(v1).equal(v1 / v1.length()));
 
 
@@ -336,143 +338,143 @@ BOOST_AUTO_TEST_CASE(vec_test_sse_vec32_consistent)
   Vec3f32SSE v3(3.4f, 4.2f, 10.5f), v4(3.1f, 0.1f, -50.4f);
   Vec3f32 v12 = v1 + v2;
   Vec3f32SSE v34 = v3 + v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 - v2;
   v34 = v3 - v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 * v2;
   v34 = v3 * v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 / v2;
   v34 = v3 / v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   float t = 1234.433f;
   v12 = v1 + t;
   v34 = v3 + t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 - t;
   v34 = v3 - t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 * t;
   v34 = v3 * t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 / t;
   v34 = v3 / t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = v1; v12 += v2;
   v34 = v3; v34 += v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 -= v2;
   v34 = v3; v34 -= v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 *= v2;
   v34 = v3; v34 *= v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 /= v2;
   v34 = v3; v34 /= v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 += t;
   v34 = v3; v34 += t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 -= t;
   v34 = v3; v34 -= t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 *= t;
   v34 = v3; v34 *= t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 /= t;
   v34 = v3; v34 /= t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = -v1;
   v34 = -v3;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = v1.cross(v2);
   v34 = v3.cross(v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
-  BOOST_CHECK(abs(v1.dot(v2) - v3.dot(v4)) < 1e-5);
+  BOOST_CHECK(std::abs(v1.dot(v2) - v3.dot(v4)) < 1e-5);
 
   v12 = min(v1, v2);
   v34 = min(v3, v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = max(v1, v2);
   v34 = max(v3, v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = abs(v2);
   v34 = abs(v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   Vec3f32 delta1(1e-9f, 1e-9f, 1e-9f);
   Vec3f32SSE delta2(1e-9f, 1e-9f, 1e-9f);
   BOOST_CHECK((v1 + delta1).equal(v1));
   BOOST_CHECK((v3 + delta2).equal(v3));
 
-  BOOST_CHECK(abs(v1.length() - v3.length()) < 1e-5);
-  BOOST_CHECK(abs(v1.sqrLength() - v3.sqrLength()) < 1e-5);
+  BOOST_CHECK(std::abs(v1.length() - v3.length()) < 1e-5);
+  BOOST_CHECK(std::abs(v1.sqrLength() - v3.sqrLength()) < 1e-5);
  
   v12 = v1; v12.negate();
   v34 = v3; v34.negate();
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = v1; v12.normalize();
   v34 = v3; v34.normalize();
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   
   v12 = normalize(v1);
   v34 = normalize(v3);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);  
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);  
 }
 
 BOOST_AUTO_TEST_CASE(vec_test_sse_vec64_consistent)
@@ -484,143 +486,143 @@ BOOST_AUTO_TEST_CASE(vec_test_sse_vec64_consistent)
   Vec3f64SSE v3(3.4, 4.2, 10.5), v4(3.1, 0.1, -50.4);
   Vec3f64 v12 = v1 + v2;
   Vec3f64SSE v34 = v3 + v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 - v2;
   v34 = v3 - v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 * v2;
   v34 = v3 * v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 / v2;
   v34 = v3 / v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   double t = 1234.433;
   v12 = v1 + t;
   v34 = v3 + t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 - t;
   v34 = v3 - t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 * t;
   v34 = v3 * t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1 / t;
   v34 = v3 / t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = v1; v12 += v2;
   v34 = v3; v34 += v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 -= v2;
   v34 = v3; v34 -= v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 *= v2;
   v34 = v3; v34 *= v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 /= v2;
   v34 = v3; v34 /= v4;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 += t;
   v34 = v3; v34 += t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 -= t;
   v34 = v3; v34 -= t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 *= t;
   v34 = v3; v34 *= t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = v1; v12 /= t;
   v34 = v3; v34 /= t;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = -v1;
   v34 = -v3;
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = v1.cross(v2);
   v34 = v3.cross(v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
-  BOOST_CHECK(abs(v1.dot(v2) - v3.dot(v4)) < 1e-5);
+  BOOST_CHECK(std::abs(v1.dot(v2) - v3.dot(v4)) < 1e-5);
 
   v12 = min(v1, v2);
   v34 = min(v3, v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   v12 = max(v1, v2);
   v34 = max(v3, v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = abs(v2);
   v34 = abs(v4);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   Vec3f64 delta1(1e-15, 1e-15, 1e-15);
   Vec3f64SSE delta2(1e-15, 1e-15, 1e-15);
   BOOST_CHECK((v1 + delta1).equal(v1));
   BOOST_CHECK((v3 + delta2).equal(v3));
 
-  BOOST_CHECK(abs(v1.length() - v3.length()) < 1e-5);
-  BOOST_CHECK(abs(v1.sqrLength() - v3.sqrLength()) < 1e-5);
+  BOOST_CHECK(std::abs(v1.length() - v3.length()) < 1e-5);
+  BOOST_CHECK(std::abs(v1.sqrLength() - v3.sqrLength()) < 1e-5);
  
   v12 = v1; v12.negate();
   v34 = v3; v34.negate();
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
 
   v12 = v1; v12.normalize();
   v34 = v3; v34.normalize();
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);
   
   v12 = normalize(v1);
   v34 = normalize(v3);
-  BOOST_CHECK(abs(v12[0] - v34[0]) < 1e-5);
-  BOOST_CHECK(abs(v12[1] - v34[1]) < 1e-5);
-  BOOST_CHECK(abs(v12[2] - v34[2]) < 1e-5);  
+  BOOST_CHECK(std::abs(v12[0] - v34[0]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[1] - v34[1]) < 1e-5);
+  BOOST_CHECK(std::abs(v12[2] - v34[2]) < 1e-5);  
 }
 
 #endif
@@ -628,15 +630,13 @@ BOOST_AUTO_TEST_CASE(vec_test_sse_vec64_consistent)
 BOOST_AUTO_TEST_CASE(morton)
 {
   AABB bbox(Vec3f(0, 0, 0), Vec3f(1000, 1000, 1000));
-  morton_functor<boost::dynamic_bitset<> > F1(bbox, 10);
-  morton_functor<boost::dynamic_bitset<> > F2(bbox, 20);
-  morton_functor<FCL_UINT64> F3(bbox);
-  morton_functor<FCL_UINT32> F4(bbox);
+  morton_functor<std::bitset<30>> F1(bbox);
+  morton_functor<std::bitset<60>> F2(bbox);
+  morton_functor<FCL_UINT64> F3(bbox); // 60 bits
+  morton_functor<FCL_UINT32> F4(bbox); // 30 bits
 
   Vec3f p(254, 873, 674);
-  std::cout << std::hex << F1(p).to_ulong() << std::endl;
-  std::cout << std::hex << F2(p).to_ulong() << std::endl;
-  std::cout << std::hex << F3(p) << std::endl;
-  std::cout << std::hex << F4(p) << std::endl;
-  
+
+  BOOST_CHECK(F1(p).to_ulong() == F4(p));
+  BOOST_CHECK(F2(p).to_ullong() == F3(p));
 }
