@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -37,8 +38,10 @@
 #define BOOST_TEST_MODULE "FCL_OCTOMAP"
 #include <boost/test/unit_test.hpp>
 
+#include "fcl/config.h"
 #include "fcl/octree.h"
 #include "fcl/traversal/traversal_node_octree.h"
+#include "fcl/collision.h"
 #include "fcl/broadphase/broadphase.h"
 #include "fcl/shape/geometric_shape_to_BVH_model.h"
 #include "fcl/math/transform.h"
@@ -87,6 +90,10 @@ void octomap_cost_test(double env_scale, std::size_t env_size, std::size_t num_m
 /// @brief Octomap distance with an environment with 3 * env_size objects
 void octomap_distance_test(double env_scale, std::size_t env_size, bool use_mesh, bool use_mesh_octomap);
 
+/// @brief Octomap collision with an environment mesh with 3 * env_size objects, asserting that correct triangle ids
+/// are returned when performing collision tests
+void octomap_collision_test_mesh_triangle_id(double env_scale, std::size_t env_size, std::size_t num_max_contacts);
+
 
 template<typename BV>
 void octomap_collision_test_BVH(std::size_t n, bool exhaustive);
@@ -117,18 +124,38 @@ BOOST_AUTO_TEST_CASE(test_octomap_collision)
 
 BOOST_AUTO_TEST_CASE(test_octomap_collision_mesh)
 {
+#ifdef FCL_BUILD_TYPE_DEBUG
+  octomap_collision_test(200, 10, false, 10, true, true);
+  octomap_collision_test(200, 100, false, 10, true, true);
+  octomap_collision_test(200, 10, true, 1, true, true);
+  octomap_collision_test(200, 100, true, 1, true, true);
+#else
   octomap_collision_test(200, 100, false, 10, true, true);
   octomap_collision_test(200, 1000, false, 10, true, true);
   octomap_collision_test(200, 100, true, 1, true, true);
   octomap_collision_test(200, 1000, true, 1, true, true);
+#endif
 }
 
+BOOST_AUTO_TEST_CASE(test_octomap_collision_mesh_triangle_id)
+{
+  octomap_collision_test_mesh_triangle_id(1, 30, 100000);
+}
+
+
 BOOST_AUTO_TEST_CASE(test_octomap_collision_mesh_octomap_box)
 {
+#ifdef FCL_BUILD_TYPE_DEBUG
+  octomap_collision_test(200, 10, false, 10, true, false);
+  octomap_collision_test(200, 100, false, 10, true, false);
+  octomap_collision_test(200, 10, true, 1, true, false);
+  octomap_collision_test(200, 100, true, 1, true, false);
+#else
   octomap_collision_test(200, 100, false, 10, true, false);
   octomap_collision_test(200, 1000, false, 10, true, false);
   octomap_collision_test(200, 100, true, 1, true, false);
   octomap_collision_test(200, 1000, true, 1, true, false);
+#endif
 }
 
 BOOST_AUTO_TEST_CASE(test_octomap_distance)
@@ -139,17 +166,26 @@ BOOST_AUTO_TEST_CASE(test_octomap_distance)
 
 BOOST_AUTO_TEST_CASE(test_octomap_distance_mesh)
 {
+#ifdef FCL_BUILD_TYPE_DEBUG
+  octomap_distance_test(200, 5, true, true);
+  octomap_distance_test(200, 50, true, true);
+#else
   octomap_distance_test(200, 100, true, true);
   octomap_distance_test(200, 1000, true, true);
+#endif
 }
 
 BOOST_AUTO_TEST_CASE(test_octomap_distance_mesh_octomap_box)
 {
+#ifdef FCL_BUILD_TYPE_DEBUG
+  octomap_distance_test(200, 10, true, false);
+  octomap_distance_test(200, 100, true, false);
+#else
   octomap_distance_test(200, 100, true, false);
   octomap_distance_test(200, 1000, true, false);
+#endif
 }
 
-
 BOOST_AUTO_TEST_CASE(test_octomap_bvh_obb_collision_obb)
 {
   octomap_collision_test_BVH<OBB>(5, false);
@@ -168,7 +204,11 @@ BOOST_AUTO_TEST_CASE(test_octomap_bvh_obb_d_distance_obb)
 
 BOOST_AUTO_TEST_CASE(test_octomap_bvh_kios_d_distance_kios)
 {
+#ifdef FCL_BUILD_TYPE_DEBUG
+  octomap_distance_test_BVH<kIOS>(2);
+#else
   octomap_distance_test_BVH<kIOS>(5);
+#endif
 }
 
 template<typename BV>
@@ -180,14 +220,14 @@ void octomap_collision_test_BVH(std::size_t n, bool exhaustive)
   loadOBJFile((path / "env.obj").string().c_str(), p1, t1);
 
   BVHModel<BV>* m1 = new BVHModel<BV>();
-  boost::shared_ptr<CollisionGeometry> m1_ptr(m1);
+  std::shared_ptr<CollisionGeometry> m1_ptr(m1);
 
   m1->beginModel();
   m1->addSubModel(p1, t1);
   m1->endModel();
 
-  OcTree* tree = new OcTree(boost::shared_ptr<const octomap::OcTree>(generateOcTree()));
-  boost::shared_ptr<CollisionGeometry> tree_ptr(tree);
+  OcTree* tree = new OcTree(std::shared_ptr<const octomap::OcTree>(generateOcTree()));
+  std::shared_ptr<CollisionGeometry> tree_ptr(tree);
 
   std::vector<Transform3f> transforms;
   FCL_REAL extents[] = {-10, -10, 10, 10, 10, 10};
@@ -246,14 +286,14 @@ void octomap_distance_test_BVH(std::size_t n)
   loadOBJFile((path / "env.obj").string().c_str(), p1, t1);
 
   BVHModel<BV>* m1 = new BVHModel<BV>();
-  boost::shared_ptr<CollisionGeometry> m1_ptr(m1);
+  std::shared_ptr<CollisionGeometry> m1_ptr(m1);
 
   m1->beginModel();
   m1->addSubModel(p1, t1);
   m1->endModel();
 
-  OcTree* tree = new OcTree(boost::shared_ptr<octomap::OcTree>(generateOcTree()));
-  boost::shared_ptr<CollisionGeometry> tree_ptr(tree);
+  OcTree* tree = new OcTree(std::shared_ptr<octomap::OcTree>(generateOcTree()));
+  std::shared_ptr<CollisionGeometry> tree_ptr(tree);
 
   std::vector<Transform3f> transforms;
   FCL_REAL extents[] = {-10, -10, 10, 10, 10, 10};
@@ -302,8 +342,8 @@ void octomap_cost_test(double env_scale, std::size_t env_size, std::size_t num_m
   else
     generateEnvironments(env, env_scale, env_size);
 
-  OcTree* tree = new OcTree(boost::shared_ptr<const octomap::OcTree>(generateOcTree()));
-  CollisionObject tree_obj((boost::shared_ptr<CollisionGeometry>(tree)));
+  OcTree* tree = new OcTree(std::shared_ptr<const octomap::OcTree>(generateOcTree()));
+  CollisionObject tree_obj((std::shared_ptr<CollisionGeometry>(tree)));
 
   DynamicAABBTreeCollisionManager* manager = new DynamicAABBTreeCollisionManager();
   manager->registerObjects(env);
@@ -422,8 +462,8 @@ void octomap_collision_test(double env_scale, std::size_t env_size, bool exhaust
   else
     generateEnvironments(env, env_scale, env_size);
 
-  OcTree* tree = new OcTree(boost::shared_ptr<const octomap::OcTree>(generateOcTree()));
-  CollisionObject tree_obj((boost::shared_ptr<CollisionGeometry>(tree)));
+  OcTree* tree = new OcTree(std::shared_ptr<const octomap::OcTree>(generateOcTree()));
+  CollisionObject tree_obj((std::shared_ptr<CollisionGeometry>(tree)));
 
   DynamicAABBTreeCollisionManager* manager = new DynamicAABBTreeCollisionManager();
   manager->registerObjects(env);
@@ -511,6 +551,30 @@ void octomap_collision_test(double env_scale, std::size_t env_size, bool exhaust
   std::cout << "Note: octomap may need more collides when using mesh, because octomap collision uses box primitive inside" << std::endl;
 }
 
+void octomap_collision_test_mesh_triangle_id(double env_scale, std::size_t env_size, std::size_t num_max_contacts)
+{
+  std::vector<CollisionObject*> env;
+  generateEnvironmentsMesh(env, env_scale, env_size);
+
+  OcTree* tree = new OcTree(std::shared_ptr<const octomap::OcTree>(generateOcTree()));
+  CollisionObject tree_obj((std::shared_ptr<CollisionGeometry>(tree)));
+
+  std::vector<CollisionObject*> boxes;
+  generateBoxesFromOctomap(boxes, *tree);
+  for(std::vector<CollisionObject*>::const_iterator cit = env.begin();
+      cit != env.end(); ++cit)
+  {
+    fcl::CollisionRequest req(num_max_contacts, true);
+    fcl::CollisionResult cResult;
+    fcl::collide(&tree_obj, *cit, req, cResult);
+    for(std::size_t index=0; index<cResult.numContacts(); ++index)
+    {
+      const Contact& contact = cResult.getContact(index);
+      const fcl::BVHModel<fcl::OBBRSS>* surface = static_cast<const fcl::BVHModel<fcl::OBBRSS>*> (contact.o2);
+      BOOST_CHECK(surface->num_tris > contact.b2);
+    }
+  }
+}
 
 void octomap_distance_test(double env_scale, std::size_t env_size, bool use_mesh, bool use_mesh_octomap)
 {
@@ -521,8 +585,8 @@ void octomap_distance_test(double env_scale, std::size_t env_size, bool use_mesh
   else
     generateEnvironments(env, env_scale, env_size);
 
-  OcTree* tree = new OcTree(boost::shared_ptr<const octomap::OcTree>(generateOcTree()));
-  CollisionObject tree_obj((boost::shared_ptr<CollisionGeometry>(tree)));
+  OcTree* tree = new OcTree(std::shared_ptr<const octomap::OcTree>(generateOcTree()));
+  CollisionObject tree_obj((std::shared_ptr<CollisionGeometry>(tree)));
 
   DynamicAABBTreeCollisionManager* manager = new DynamicAABBTreeCollisionManager();
   manager->registerObjects(env);
@@ -602,7 +666,7 @@ void octomap_distance_test(double env_scale, std::size_t env_size, bool use_mesh
 
 void generateBoxesFromOctomap(std::vector<CollisionObject*>& boxes, OcTree& tree)
 {
-  std::vector<boost::array<FCL_REAL, 6> > boxes_ = tree.toBoxes();
+  std::vector<std::array<FCL_REAL, 6> > boxes_ = tree.toBoxes();
 
   for(std::size_t i = 0; i < boxes_.size(); ++i)
   {
@@ -616,7 +680,7 @@ void generateBoxesFromOctomap(std::vector<CollisionObject*>& boxes, OcTree& tree
     Box* box = new Box(size, size, size);
     box->cost_density = cost;
     box->threshold_occupied = threshold;
-    CollisionObject* obj = new CollisionObject(boost::shared_ptr<CollisionGeometry>(box), Transform3f(Vec3f(x, y, z)));
+    CollisionObject* obj = new CollisionObject(std::shared_ptr<CollisionGeometry>(box), Transform3f(Vec3f(x, y, z)));
     boxes.push_back(obj);
   }
 
@@ -633,21 +697,21 @@ void generateEnvironments(std::vector<CollisionObject*>& env, double env_scale,
   for(std::size_t i = 0; i < n; ++i)
   {
     Box* box = new Box(5, 10, 20);
-    env.push_back(new CollisionObject(boost::shared_ptr<CollisionGeometry>(box), transforms[i]));
+    env.push_back(new CollisionObject(std::shared_ptr<CollisionGeometry>(box), transforms[i]));
   }
 
   generateRandomTransforms(extents, transforms, n);
   for(std::size_t i = 0; i < n; ++i)
   {
     Sphere* sphere = new Sphere(30);
-    env.push_back(new CollisionObject(boost::shared_ptr<CollisionGeometry>(sphere), transforms[i]));
+    env.push_back(new CollisionObject(std::shared_ptr<CollisionGeometry>(sphere), transforms[i]));
   }
 
   generateRandomTransforms(extents, transforms, n);
   for(std::size_t i = 0; i < n; ++i)
   {
     Cylinder* cylinder = new Cylinder(10, 40);
-    env.push_back(new CollisionObject(boost::shared_ptr<CollisionGeometry>(cylinder), transforms[i]));
+    env.push_back(new CollisionObject(std::shared_ptr<CollisionGeometry>(cylinder), transforms[i]));
   }
 
 }
@@ -655,7 +719,7 @@ void generateEnvironments(std::vector<CollisionObject*>& env, double env_scale,
 
 void generateBoxesFromOctomapMesh(std::vector<CollisionObject*>& boxes, OcTree& tree)
 {
-  std::vector<boost::array<FCL_REAL, 6> > boxes_ = tree.toBoxes();
+  std::vector<std::array<FCL_REAL, 6> > boxes_ = tree.toBoxes();
 
   for(std::size_t i = 0; i < boxes_.size(); ++i)
   {
@@ -671,7 +735,7 @@ void generateBoxesFromOctomapMesh(std::vector<CollisionObject*>& boxes, OcTree&
     generateBVHModel(*model, box, Transform3f());
     model->cost_density = cost;
     model->threshold_occupied = threshold;
-    CollisionObject* obj = new CollisionObject(boost::shared_ptr<CollisionGeometry>(model), Transform3f(Vec3f(x, y, z)));
+    CollisionObject* obj = new CollisionObject(std::shared_ptr<CollisionGeometry>(model), Transform3f(Vec3f(x, y, z)));
     boxes.push_back(obj);    
   }
 
@@ -689,7 +753,7 @@ void generateEnvironmentsMesh(std::vector<CollisionObject*>& env, double env_sca
   {
     BVHModel<OBBRSS>* model = new BVHModel<OBBRSS>();
     generateBVHModel(*model, box, Transform3f());
-    env.push_back(new CollisionObject(boost::shared_ptr<CollisionGeometry>(model), transforms[i]));
+    env.push_back(new CollisionObject(std::shared_ptr<CollisionGeometry>(model), transforms[i]));
   }
 
   generateRandomTransforms(extents, transforms, n);
@@ -698,7 +762,7 @@ void generateEnvironmentsMesh(std::vector<CollisionObject*>& env, double env_sca
   {
     BVHModel<OBBRSS>* model = new BVHModel<OBBRSS>();
     generateBVHModel(*model, sphere, Transform3f(), 16, 16);
-    env.push_back(new CollisionObject(boost::shared_ptr<CollisionGeometry>(model), transforms[i]));
+    env.push_back(new CollisionObject(std::shared_ptr<CollisionGeometry>(model), transforms[i]));
   }
 
   generateRandomTransforms(extents, transforms, n);
@@ -707,7 +771,7 @@ void generateEnvironmentsMesh(std::vector<CollisionObject*>& env, double env_sca
   {
     BVHModel<OBBRSS>* model = new BVHModel<OBBRSS>();
     generateBVHModel(*model, cylinder, Transform3f(), 16, 16);
-    env.push_back(new CollisionObject(boost::shared_ptr<CollisionGeometry>(model), transforms[i]));
+    env.push_back(new CollisionObject(std::shared_ptr<CollisionGeometry>(model), transforms[i]));
   }
 }
 
