@@ -1,7 +1,8 @@
 /*
  * Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -141,6 +142,96 @@ BOOST_AUTO_TEST_CASE(consistency_distance_spheresphere_libccd)
   }
 }
 
+BOOST_AUTO_TEST_CASE(consistency_distance_ellipsoidellipsoid_libccd)
+{
+  Ellipsoid s1(20, 40, 50);
+  Ellipsoid s2(10, 10, 10);
+  BVHModel<RSS> s1_rss;
+  BVHModel<RSS> s2_rss;
+
+  generateBVHModel(s1_rss, s1, Transform3f(), 16, 16);
+  generateBVHModel(s2_rss, s2, Transform3f(), 16, 16);
+
+  DistanceRequest request;
+  DistanceResult res, res1;
+
+  Transform3f pose;
+
+  pose.setTranslation(Vec3f(40, 0, 0));
+
+  res.clear(); res1.clear();
+  distance(&s1, Transform3f(), &s2, pose, request, res);
+  distance(&s1_rss, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+  res1.clear();
+  distance(&s1, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+  res1.clear();
+  distance(&s1_rss, Transform3f(), &s2, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+  for(std::size_t i = 0; i < 10; ++i)
+  {
+    Transform3f t;
+    generateRandomTransform(extents, t);
+
+    Transform3f pose1(t);
+    Transform3f pose2 = t * pose;
+
+    res.clear(); res1.clear();
+    distance(&s1, pose1, &s2, pose2, request, res);
+    distance(&s1_rss, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+    res1.clear();
+    distance(&s1, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+    res1.clear();
+    distance(&s1_rss, pose1, &s2, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+  }
+
+  pose.setTranslation(Vec3f(30.1, 0, 0));
+
+  res.clear(), res1.clear();
+  distance(&s1, Transform3f(), &s2, pose, request, res);
+  distance(&s1_rss, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+  res1.clear();
+  distance(&s1, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+  res1.clear();
+  distance(&s1_rss, Transform3f(), &s2, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+  for(std::size_t i = 0; i < 10; ++i)
+  {
+    Transform3f t;
+    generateRandomTransform(extents, t);
+
+    Transform3f pose1(t);
+    Transform3f pose2 = t * pose;
+
+    res.clear(); res1.clear();
+    distance(&s1, pose1, &s2, pose2, request, res);
+    distance(&s1_rss, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+    res1.clear();
+    distance(&s1, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+    res1.clear();
+    distance(&s1_rss, pose1, &s2, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+  }
+}
+
 BOOST_AUTO_TEST_CASE(consistency_distance_boxbox_libccd)
 {
   Box s1(20, 40, 50);
@@ -507,6 +598,99 @@ BOOST_AUTO_TEST_CASE(consistency_distance_spheresphere_GJK)
   }
 }
 
+BOOST_AUTO_TEST_CASE(consistency_distance_ellipsoidellipsoid_GJK)
+{
+  Ellipsoid s1(20, 40, 50);
+  Ellipsoid s2(10, 10, 10);
+  BVHModel<RSS> s1_rss;
+  BVHModel<RSS> s2_rss;
+
+  generateBVHModel(s1_rss, s1, Transform3f(), 16, 16);
+  generateBVHModel(s2_rss, s2, Transform3f(), 16, 16);
+
+  DistanceRequest request;
+  request.gjk_solver_type = GST_INDEP;
+  DistanceResult res, res1;
+
+  Transform3f pose;
+
+  pose.setTranslation(Vec3f(40, 0, 0));
+
+  res.clear(); res1.clear();
+  distance(&s1, Transform3f(), &s2, pose, request, res);
+  distance(&s1_rss, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+  res1.clear();
+  distance(&s1, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+  res1.clear();
+  distance(&s1_rss, Transform3f(), &s2, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+
+  for(std::size_t i = 0; i < 10; ++i)
+  {
+    Transform3f t;
+    generateRandomTransform(extents, t);
+
+    Transform3f pose1(t);
+    Transform3f pose2 = t * pose;
+
+    res.clear(); res1.clear();
+    distance(&s1, pose1, &s2, pose2, request, res);
+    distance(&s1_rss, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+    res1.clear();
+    distance(&s1, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+
+    res1.clear();
+    distance(&s1_rss, pose1, &s2, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 0.05);
+  }
+
+  pose.setTranslation(Vec3f(30.1, 0, 0));
+
+  res.clear(); res1.clear();
+  distance(&s1, Transform3f(), &s2, pose, request, res);
+  distance(&s1_rss, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+  res1.clear();
+  distance(&s1, Transform3f(), &s2_rss, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+  res1.clear();
+  distance(&s1_rss, Transform3f(), &s2, pose, request, res1);
+  BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 4);
+
+
+  for(std::size_t i = 0; i < 10; ++i)
+  {
+    Transform3f t;
+    generateRandomTransform(extents, t);
+
+    Transform3f pose1(t);
+    Transform3f pose2 = t * pose;
+
+    res.clear(); res1.clear();
+    distance(&s1, pose1, &s2, pose2, request, res);
+    distance(&s1_rss, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+    res1.clear();
+    distance(&s1, pose1, &s2_rss, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 2);
+
+    res1.clear();
+    distance(&s1_rss, pose1, &s2, pose2, request, res1);
+    BOOST_CHECK(fabs(res1.min_distance - res.min_distance) / res.min_distance < 4);
+  }
+}
+
 BOOST_AUTO_TEST_CASE(consistency_distance_boxbox_GJK)
 {
   Box s1(20, 40, 50);
@@ -519,6 +703,7 @@ BOOST_AUTO_TEST_CASE(consistency_distance_boxbox_GJK)
   generateBVHModel(s2_rss, s2, Transform3f());
 
   DistanceRequest request;
+  request.gjk_solver_type = GST_INDEP;
   DistanceResult res, res1;
 
   Transform3f pose;
@@ -610,6 +795,7 @@ BOOST_AUTO_TEST_CASE(consistency_distance_cylindercylinder_GJK)
   generateBVHModel(s2_rss, s2, Transform3f(), 16, 16);
   
   DistanceRequest request;
+  request.gjk_solver_type = GST_INDEP;
   DistanceResult res, res1;
 
   Transform3f pose;
@@ -710,6 +896,7 @@ BOOST_AUTO_TEST_CASE(consistency_distance_conecone_GJK)
   generateBVHModel(s2_rss, s2, Transform3f(), 16, 16);
 
   DistanceRequest request;
+  request.gjk_solver_type = GST_INDEP;
   DistanceResult res, res1;
 
   Transform3f pose;
@@ -877,7 +1064,7 @@ BOOST_AUTO_TEST_CASE(consistency_collision_spheresphere_libccd)
   result.clear();
   res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
   BOOST_CHECK_FALSE(res);
-  
+
   pose.setTranslation(Vec3f(30, 0, 0));
   pose_aabb.setTranslation(Vec3f(30, 0, 0));
   pose_obb.setTranslation(Vec3f(30, 0, 0));
@@ -909,7 +1096,7 @@ BOOST_AUTO_TEST_CASE(consistency_collision_spheresphere_libccd)
   result.clear();
   res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
   BOOST_CHECK_FALSE(res);
-  
+
   pose.setTranslation(Vec3f(29.9, 0, 0));
   pose_aabb.setTranslation(Vec3f(29.8, 0, 0)); // 29.9 fails, result depends on mesh precision
   pose_obb.setTranslation(Vec3f(29.8, 0, 0)); // 29.9 fails, result depends on mesh precision
@@ -1008,6 +1195,225 @@ BOOST_AUTO_TEST_CASE(consistency_collision_spheresphere_libccd)
   BOOST_CHECK_FALSE(res);
 }
 
+BOOST_AUTO_TEST_CASE(consistency_collision_ellipsoidellipsoid_libccd)
+{
+  Ellipsoid s1(20, 40, 50);
+  Ellipsoid s2(10, 10, 10);
+  BVHModel<AABB> s1_aabb;
+  BVHModel<AABB> s2_aabb;
+  BVHModel<OBB> s1_obb;
+  BVHModel<OBB> s2_obb;
+
+  generateBVHModel(s1_aabb, s1, Transform3f(), 16, 16);
+  generateBVHModel(s2_aabb, s2, Transform3f(), 16, 16);
+  generateBVHModel(s1_obb, s1, Transform3f(), 16, 16);
+  generateBVHModel(s2_obb, s2, Transform3f(), 16, 16);
+
+  CollisionRequest request;
+  CollisionResult result;
+
+  bool res;
+
+  Transform3f pose, pose_aabb, pose_obb;
+
+
+  // s2 is within s1
+  // both are shapes --> collision
+  // s1 is shape, s2 is mesh --> in collision
+  // s1 is mesh, s2 is shape --> collision free
+  // all are reasonable
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+
+  pose.setTranslation(Vec3f(40, 0, 0));
+  pose_aabb.setTranslation(Vec3f(40, 0, 0));
+  pose_obb.setTranslation(Vec3f(40, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+
+  pose.setTranslation(Vec3f(30, 0, 0));
+  pose_aabb.setTranslation(Vec3f(30, 0, 0));
+  pose_obb.setTranslation(Vec3f(30, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);  // libccd cannot detect collision when two ellipsoid is exactly touching each other
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, Transform3f(Vec3f(29.999, 0, 0)), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+
+  pose.setTranslation(Vec3f(29.9, 0, 0));
+  pose_aabb.setTranslation(Vec3f(29.9, 0, 0)); // 29.9 fails, result depends on mesh precision
+  pose_obb.setTranslation(Vec3f(29.9, 0, 0)); // 29.9 fails, result depends on mesh precision
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+
+  pose.setTranslation(Vec3f(-29.9, 0, 0));
+  pose_aabb.setTranslation(Vec3f(-29.9, 0, 0));
+  pose_obb.setTranslation(Vec3f(-29.9, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+
+  pose.setTranslation(Vec3f(-30, 0, 0));
+  pose_aabb.setTranslation(Vec3f(-30, 0, 0));
+  pose_obb.setTranslation(Vec3f(-30, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+}
+
 BOOST_AUTO_TEST_CASE(consistency_collision_boxbox_libccd)
 {
   Box s1(20, 40, 50);
@@ -1577,7 +1983,7 @@ BOOST_AUTO_TEST_CASE(consistency_collision_spheresphere_GJK)
   result.clear();
   res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
   BOOST_CHECK_FALSE(res);
-  
+
   pose.setTranslation(Vec3f(30, 0, 0));
   pose_aabb.setTranslation(Vec3f(30, 0, 0));
   pose_obb.setTranslation(Vec3f(30, 0, 0));
@@ -1609,7 +2015,7 @@ BOOST_AUTO_TEST_CASE(consistency_collision_spheresphere_GJK)
   result.clear();
   res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
   BOOST_CHECK_FALSE(res);
-  
+
   pose.setTranslation(Vec3f(29.9, 0, 0));
   pose_aabb.setTranslation(Vec3f(29.8, 0, 0)); // 29.9 fails, result depends on mesh precision
   pose_obb.setTranslation(Vec3f(29.8, 0, 0)); // 29.9 fails, result depends on mesh precision
@@ -1708,6 +2114,225 @@ BOOST_AUTO_TEST_CASE(consistency_collision_spheresphere_GJK)
   BOOST_CHECK_FALSE(res);
 }
 
+BOOST_AUTO_TEST_CASE(consistency_collision_ellipsoidellipsoid_GJK)
+{
+  Ellipsoid s1(20, 40, 50);
+  Ellipsoid s2(10, 10, 10);
+  BVHModel<AABB> s1_aabb;
+  BVHModel<AABB> s2_aabb;
+  BVHModel<OBB> s1_obb;
+  BVHModel<OBB> s2_obb;
+
+  generateBVHModel(s1_aabb, s1, Transform3f(), 16, 16);
+  generateBVHModel(s2_aabb, s2, Transform3f(), 16, 16);
+  generateBVHModel(s1_obb, s1, Transform3f(), 16, 16);
+  generateBVHModel(s2_obb, s2, Transform3f(), 16, 16);
+
+  CollisionRequest request;
+  request.gjk_solver_type = GST_INDEP;
+
+  CollisionResult result;
+
+  bool res;
+
+  Transform3f pose, pose_aabb, pose_obb;
+
+
+  // s2 is within s1
+  // both are shapes --> collision
+  // s1 is shape, s2 is mesh --> in collision
+  // s1 is mesh, s2 is shape --> collision free
+  // all are reasonable
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+
+  pose.setTranslation(Vec3f(40, 0, 0));
+  pose_aabb.setTranslation(Vec3f(40, 0, 0));
+  pose_obb.setTranslation(Vec3f(40, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+
+  pose.setTranslation(Vec3f(30, 0, 0));
+  pose_aabb.setTranslation(Vec3f(30, 0, 0));
+  pose_obb.setTranslation(Vec3f(30, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+
+  pose.setTranslation(Vec3f(29.9, 0, 0));
+  pose_aabb.setTranslation(Vec3f(29.9, 0, 0));
+  pose_obb.setTranslation(Vec3f(29.9, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+
+
+  pose.setTranslation(Vec3f(-29.9, 0, 0));
+  pose_aabb.setTranslation(Vec3f(-29.9, 0, 0));
+  pose_obb.setTranslation(Vec3f(-29.9, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+
+  pose.setTranslation(Vec3f(-30, 0, 0));
+  pose_aabb.setTranslation(Vec3f(-30, 0, 0));
+  pose_obb.setTranslation(Vec3f(-30, 0, 0));
+
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK(res);
+  result.clear();
+  res = (collide(&s2_aabb, pose_aabb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2_obb, pose_obb, &s1, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_aabb, pose_aabb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1, Transform3f(), &s2_obb, pose_obb, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_aabb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s2, pose, &s1_obb, Transform3f(), request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+  result.clear();
+  res = (collide(&s1_aabb, Transform3f(), &s2, pose, request, result) > 0);
+  BOOST_CHECK_FALSE(res);
+}
+
 BOOST_AUTO_TEST_CASE(consistency_collision_boxbox_GJK)
 {
   Box s1(20, 40, 50);
