@@ -1,7 +1,8 @@
 /*
  *  Software License Agreement (BSD License)
  *
- *  Copyright (c) 2011, Willow Garage, Inc.
+ *  Copyright (c) 2011-2014, Willow Garage, Inc.
+ *  Copyright (c) 2014-2016, Open Source Robotics Foundation
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *   * Neither the name of Willow Garage, Inc. nor the names of its
+ *   * Neither the name of Open Source Robotics Foundation nor the names of its
  *     contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission.
  *
@@ -37,14 +38,11 @@
 #define BOOST_TEST_MODULE "FCL_SPHERE_CAPSULE"
 #include <boost/test/unit_test.hpp>
 
+#include "fcl/math/constants.h"
 #include "fcl/collision.h"
 #include "fcl/shape/geometric_shapes.h"
 #include "fcl/narrowphase/narrowphase.h"
 
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 using namespace fcl;
 
 BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_z)
@@ -58,7 +56,7 @@ BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_z)
 	Capsule capsule (50, 200.);
 	Transform3f capsule_transform (Vec3f (0., 0., 200));
 
-	BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL, NULL, NULL));
+  BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL));
 }
 
 BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_z_negative)
@@ -72,7 +70,7 @@ BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_z_negative)
 	Capsule capsule (50, 200.);
 	Transform3f capsule_transform (Vec3f (0., 0., -200));
 
-	BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL, NULL, NULL));
+  BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL));
 }
 
 BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_x)
@@ -86,7 +84,7 @@ BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_x)
 	Capsule capsule (50, 200.);
 	Transform3f capsule_transform (Vec3f (150., 0., 0.));
 
-	BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL, NULL, NULL));
+  BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL));
 }
 
 BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_capsule_rotated)
@@ -99,33 +97,35 @@ BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_separated_capsule_rotated)
 
 	Capsule capsule (50, 200.);
 	Matrix3f rotation;
-	rotation.setEulerZYX (M_PI * 0.5, 0., 0.);
+	rotation.setEulerZYX (constants::pi * 0.5, 0., 0.);
 	Transform3f capsule_transform (rotation, Vec3f (150., 0., 0.));
 
-	BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL, NULL, NULL));
+  BOOST_CHECK (!solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, NULL));
 }
 
 BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_penetration_z)
 {
-	GJKSolver_libccd solver;
+  GJKSolver_libccd solver;
 
-	Sphere sphere1 (50);
-	Transform3f sphere1_transform;
-	sphere1_transform.setTranslation (Vec3f (0., 0., -50));
+  Sphere sphere1 (50);
+  Transform3f sphere1_transform;
+  sphere1_transform.setTranslation (Vec3f (0., 0., -50));
 
-	Capsule capsule (50, 200.);
-	Transform3f capsule_transform (Vec3f (0., 0., 125));
+  Capsule capsule (50, 200.);
+  Transform3f capsule_transform (Vec3f (0., 0., 125));
 
-	FCL_REAL penetration = 0.;
-	Vec3f contact_point;
-	Vec3f normal;
+  std::vector<ContactPoint> contacts;
 
-	bool is_intersecting = solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, &contact_point, &penetration, &normal);
+  bool is_intersecting = solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, &contacts);
 
-	BOOST_CHECK (is_intersecting);
-	BOOST_CHECK (penetration == 25.);
-	BOOST_CHECK (Vec3f (0., 0., 1.).equal(normal));
-	BOOST_CHECK (Vec3f (0., 0., 0.).equal(contact_point));
+  FCL_REAL penetration = contacts[0].penetration_depth;
+  Vec3f contact_point = contacts[0].pos;
+  Vec3f normal = contacts[0].normal;
+
+  BOOST_CHECK (is_intersecting);
+  BOOST_CHECK (penetration == 25.);
+  BOOST_CHECK (Vec3f (0., 0., 1.).equal(normal));
+  BOOST_CHECK (Vec3f (0., 0., 0.).equal(contact_point));
 }
 
 BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_penetration_z_rotated)
@@ -138,14 +138,16 @@ BOOST_AUTO_TEST_CASE(Sphere_Capsule_Intersect_test_penetration_z_rotated)
 
 	Capsule capsule (50, 200.);
 	Matrix3f rotation;
-	rotation.setEulerZYX (M_PI * 0.5, 0., 0.);
+	rotation.setEulerZYX (constants::pi * 0.5, 0., 0.);
 	Transform3f capsule_transform (rotation, Vec3f (0., 50., 75));
 
-	FCL_REAL penetration = 0.;
-	Vec3f contact_point;
-	Vec3f normal;
+  std::vector<ContactPoint> contacts;
 
-	bool is_intersecting = solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, &contact_point, &penetration, &normal);
+  bool is_intersecting = solver.shapeIntersect(sphere1, sphere1_transform, capsule, capsule_transform, &contacts);
+
+  FCL_REAL penetration = contacts[0].penetration_depth;
+  Vec3f contact_point = contacts[0].pos;
+  Vec3f normal = contacts[0].normal;
 
 	BOOST_CHECK (is_intersecting);
 	BOOST_CHECK_CLOSE (25, penetration, solver.collision_tolerance);
